;----------------------------------------------------------------------------------------
; Gravacao da memoria de programa                                              26/07/2020
;
; STATUS: FUNCIONA
;
; Esta funcao recebe do programador as linhas de programa do arquivo .HEX
; no formato hexadecimal. Cada linha estara' no formato --> BBAAaaTTDD...CC onde:
;       BB - Numero de bytes de dados na linha.
;       AA - MSB do endereco da memoria de programa onde o dado sera gravado.
;       aa - LSB do endereco da memoria de programa onde o dado sera gravado.
;       TT - Tipo de dado:
;            00 - Dados de programa.
;            01 - Fim de arquivo.
;            04 - Endereco extendido: Indica que os valores dos dados sao os
;                 16 bits superiores do endereco.
;       DD - Dado que contem o codigo de maquina gerado pelo programa.
;            Podem haver varios DD numa linha. BB indica quantos. 
;       CC - Check Sum complemento de 2:  BB+aa+AA+TT+DD+DD+....+DD
;
;
;----------------------------------------------------------------------------------------
; NOTAS:
; 1 - A cada byte gravado, a funcao de verificacao 'pgmVerify' sera chamada para 
;     confirmar se o byte foi gravado corretamente e, se nao foi, serao tentadas 
;     regravacoes sucessivas no mesmo endereco (ver simbolo REC_RETRY).
;
;
;----------------------------------------------------------------------------------------
; O seguinte procedimento sera seguido nesta funcao:
;
;  1 - O modo de programacao e' desabilitado para ter certeza que o chip esta 
;      desativado.
;
;  2 - Se o simbolo 'NO_CONFIG_BITS'...
;      2.1 - ...esta definido, sera usada a funcao 'picEraseProg' que	
;               apagara somente a memoria de programa.  
;      2.2 - ...nao esta definido, sera usada a funcao 'picEraseConfig' que
;               apagara a memoria de programa e configuracao.
;
;  3 - Envia uma mensagem "ACK" ao PC para dizer que esta pronto para
;      receber o programa.
;
;  4 - Le uma linha de programa enviada pelo PC em fomtato binario.
;
;  5 - Se recebeu do PC a mensagem "EOT", termina o programa. 
;
;  6 - Recupera os dados da linha de programae coloca no buffer iniciado em
;      'dataLen'.
;
;  7 - Se recebeu "EOF" (dataType=0x01), envia "ACK" ap PC e encerra o programa.
;
;  8 - Se o tipo de dado nao e' um dado de programa (dataType!=0x00) despreza 
;      a linha atual e le a proxima linha.
;
;  9 - Verifica o CRC e, se nao estiver correto, encerra o programa.
;
; 10 - Verifica se deve gravar a memoria de configuracao.
;      10.1 - Se o simbolo 'NO_CONFIG_BITS' esta definido e o endereco recebido em
;             dataH:dataL for x040OE, despreza a linha e vai receber a proxima.
;      10.2 - Se o simbolo 'NO_CONFIG_BITS' nao esta definid0 e o endereco recebido em
;             dataH:dataL for x040OE, grava a memoria de configuracao.
;
; 11 - Verifica se deve gravar a memoria EEPROM.
;      11.1 - Se o MSB do endereco for 0x42, grava a memoria EEPROM.
;
; 12 - Posiciona o Program Counter no endereco dataH:dataL recebido na linha de programa
;      usando a funcao 'setPmAddr'. Essa funcao coloca o chip em modo de programacao.
;
; 13 - Envia os dados de programa iniciados em 'dataBytes' ao microcontrolador.
;      12.1 - Pega um byte no buffer e tenta gravar com a funcao 'picWriteData'..
;      12.2 - Verifica se gravou corretamente com a funcao 'pgmVerify'.
;             12.2.1 - Se incorreto, tenta regravar o numero de vezes especificados no
;                      simbolo 'REC_RETRY'e se a falha persistir, encerra o programa.
;   
; 14 - Se dataLen==0, vai receber a proxima linha de programa ---> item 4.
;   
; 15 - Avanca um byte na memoria de programa (INCREMENT ADDRESS).
;
; 16 - Vai gravar o proximo byte ---> item 12.1
;
;----------------------------------------------------------------------------------------
chip_ldpm      call    picStop                   ; reset
               movlw   .250                      ; tempo
               call    delay_ms                  ; tempo desligado

               ; Apaga as memorias antes de iniciar a programacao

               call    picEraseEprom                ; apaga a EEPROM sumariamente

           ifdef NO_CONFIG_BITS
               ; Isto deve apagar a memoria de programa.
               call    picEraseProg              ; apaga memoria de programa 
           else
               ; Teoricamente isto deve apagar a memoria de configuracao e
               ; a memoria de programa tambem.
               call    picEraseConfig               ; apaga memoria de configuracao 
           endif

               ; Envia um 'ACK' porque o programa do PC esta' esperando
chip_ldpm2     SERIAL_WRITE msACK                ; send 'ACK' apos receber '-wp'
               movlw   0x0D                      ; 
               call    serialPut                 ; envia \r de volta

               ; Recebe uma linha de programa do PC
               bcf    SERFLAGS, 7                   ; gambiarra para ler por tamanho
               SERIAL_READ  pgrbuff              ; recebe uma linha de programa

               ; Verifica se 'EOT' foi enviado pelo PC
               movlw   pgrbuff                   ; inicio do buffer
               movwf   FSR                       ; carrega indexador
               call    testEOT                   ; identifica EOT
               xorlw   0x00                      ; verifica EOT
               btfsc   STATUS, Z                 ; EOT???
               goto    chip_ldpm88               ; SIM --> termina este processo

               ; Recuperacao dos dados do buffer de leitura que estao no banco 1,
               ; preenchendo os registradores 'dataLen', 'dataH', 'dataL', 'dadaType' 
               ; e 'dataBytes' que estao no banco 0.
chip_ldpm2a    call    dataRecovery              ; recupera dados do buffer de leitura


              ; Verifica se e' fim de arquivo (0x00000001FF)
               movlw   0x01                      ; marca de fim de arquivo
               xorwf   dataType, w               ; verifica se e' fim de arquivo
               btfss   STATUS, Z                 ; EOF ???
               goto    chip_ldpm21               ; NAO --> continua
            ifdef LCD_REQUIRED
               call    SR_LCDCLEAR               ;
               call    msgEOF                    ;
            endif
               goto    chip_ldpm88               ; SIM --> encerra

               ; Verifica o tipo de dado antes de programar
chip_ldpm21    movf    dataType, w               ; pega o tipo de dado
               xorlw   0x00                      ; verifica se e' dado de programa
               btfss   STATUS, Z                 ; dado de programa???
               goto    chip_ldpm2                ; NAO --> ignora, nao sei o que fazer

               ; Verificacao do CRC.
               call    crcMatch                  ; verifica o CRC
               xorlw   0x00                      ; que tal?
               btfsc   STATUS, Z                 ; CRC bagual???
               goto    chip_ldpm22               ; SIM --> continua...
            ifdef LCD_REQUIRED
               call    SR_LCDCLEAR
               call    msgErro
               SET_CURSOR .2, .1, "i"
               call    msgCRC
               movf    crrec, w
               call    lcdHexa
               movlw   "/"
               call    SR_SENDW
               movf    crcalc, w
               call    lcdHexa
            endif
               SERIAL_WRITE msEOT                ; envia 'EOT'
               movlw  0x0D                       ; fim da linha
               call   serialPut                  ; encerra 
               goto    0x00


chip_ldpm22
               ; Verifica se deve gravar a memoria de configuracao
               #include "Includes\confaddr.inc"           ; segmento adicional para processo
                                                 ; de gravacao da memoria de configuracao

               ; Verifica se  dado a ser gravado na memoria EEPROM
               #include "Includes\eepromaddr.inc"         ; segmento adicional para o processo 
                                                 ; de gravacao da memoria EEPROM

               ; Posiciona memoria de programa no endereco dataH:dataL
               call    setPmAddr                 ; posiciona no endereco e retorna
                                                 ; em modo de programacao
                                                 ; arquivo --> setpmaddr.inc



               ; Envia os dados ao microcontrolador (2 a 2)
               rrf     dataLen, f                ; divide o tamanho por 2
               bcf     dataLen, 7                ; limpa sujeira do Carry
               movf    dataLen, w                ; pega o tamanho
               xorlw   0x00                      ; verifica o tamanho
               btfss   STATUS, Z                 ; tamanho e' zero???
               goto    chip_ldpm23               ; NAO --> continua
            ifdef LCD_REQUIRED
               call    SR_LCDCLEAR               ;
               call    msgData                   ;
               movlw   "0"
               call    SR_SENDW
               movlw   "0"
               call    SR_SENDW
             endif
               goto    chip_ldpm88               ; SIM --> termina prematuramente

               ; ---------------------------------------------------------------------
               ; Grava um byte no microcontrolador
               ; O byte em dataBytes esta invertido --> LSB:MSB e sera
               ; movido para  pgmr_data1:pgmr_data0 --> MSB:LSB no formato direto
               ; --------------------------------------------------------------------
chip_ldpm23    
            ifdef REC_RETRY
               movlw   REC_RETRY                 ; numero de tentativas
            else
               movlw   .5                        ; default se REC_RETRY nao foi definido
            endif
               movwf   picretry                  ; carrega contador
               movlw   dataBytes                 ; endereco dos dados
               movwf   FSR                       ; carrega o indexador
chip_ldpm3     movf    INDF, w                   ; pega 8 bits LSB
               incf    FSR, f                    ; avanca o indexador
               movwf   pgmr_data0                ; salva LSB
               movwf   bkp0                      ; salva para poder conferir depois
               movf    INDF, w                   ; pega 8 bits MSB
               incf    FSR, f                    ; avanca o indexador
               movwf   pgmr_data1                ; salva MSB
               movwf   bkp1                      ; salva para poder conferir depois
chip_ldpm3a    call    picWriteData              ; grava um byte 

               ; --------------------------------------------------------------------
               ; Verificacao da gravacao - (31/07/2020).
               ; Isto so vai funcionar se o simbolo '_VERIFY' estiver definido,
               ; caso contrario o registrador W retornara da funcao 'pgmVerify' 
               ; sempre com o valor zero, indicando que a verificacao esta' correta.
               ; --------------------------------------------------------------------
               call    pgmVerify                 ; verifica a gravacao
               xorlw   0x00                      ; confere
               btfsc   STATUS, Z                 ; certo?
               goto    chip_ldpm4                ; SIM --> continua...

               ; Erro de gravacao... tenta novamente
             ifdef LCD_REQUIRED
               SET_CURSOR .2, .1, "i"
               call    msgRetry                  ; msg="RETRY "
               movf    picretry, w               ; numero da tentativa
               call    lcdHexa                   ; mostra o numero
             endif
               movf    bkp0, w                   ; recupera LSB
               movwf   pgmr_data0                ; restaura LSB
               movf    bkp1, w                   ; recupera MSB
               movwf   pgmr_data1                ; restaura MSB
               decfsz  picretry, f               ; fim do retry???
               goto    chip_ldpm2a               ; NAO --> repete desde dataRecovery
               SERIAL_WRITE msERC                ; SIM --> erro
               movlw   0x0D                      ;
               call    serialPut                 ;
               call    picStop                   ; sai do modo de programacao
               TEMPO   .5                        ; tempo para visualizar a mensagem
               goto    0x00                      ; reset

               ; Verifica se e' fim de programacao
chip_ldpm4     decfsz  dataLen, f                ; fim da linha???
               goto    chip_ldpm4a               ; NAO --> vai avancar endereco
               goto    chip_ldpm2                ; SIM --> vai pegar outra linha

               ; Proxima posicao na memoria de programa
chip_ldpm4a    movlw   INC_ADDR                  ; INCREMENT ADDRESS
               movwf   pgmr_cmd                  ; carrega buffer de comando
               call    picSendCmd               ; envia o comando
               goto    chip_ldpm3                ; vai enviar outro byte

               ; Termina gravacao enviando 'ACK'
chip_ldpm88    call    picStop                   ; sai do modo de programacao

               movlw   .50
               call    delay_ms
               call    chipDetect
               movlw   .50
               call    delay_ms

               SERIAL_WRITE msACK                ; send 'ACK' 
               movlw   0x0D                      ; 
               call    serialPut                 ; envia \r de volta
             ifdef LCD_REQUIRED
               call    msgEOJ
               TEMPO   .2
             endif
               SERIAL_WRITE msACK                ; send 'ACK' apos receber '-wp'
               movlw   0x0D                      ; 
               call    serialPut                 ; envia \r de volta
               goto    0x00


; -----------------------------------------------------------------------------
; Funcoes usadas na opcao de gravacao de memoria do programa
; -----------------------------------------------------------------------------
               #include "Includes\datarecovery.inc"
               #include "Includes\setpmaddr.inc"
               #include "Includes\testeot.inc"
               #include "Includes\crcmatch.inc"
               #include "Includes\pgmverify.inc"
