; ------------------------------------------------------------------------------------
; Gravacao da memoria de configuracao                                       05/08/2020
;
; STATUS: FUNCIONA
;
; Grava a memoria de configuracao nos enderecos 0x2000 a 0x2007 cujo layout e':
;
;                     2000 2001 2002 2003 2004 2005 2006 2007
;                     UUuu:UUuu:UUuu:UUuu:RRrr:RRrr:DDdd:WWww 
;
; onde
;
; UUuu ......... User ID............... Por default sera gravado 0x3FFF
; RRrr ......... Reserved.............. Nao sera gravado
; DDdd ......... Device ID............. Nao sera gravado. Ver abaixo.
; WWww ......... Configuration Word.... Depende dos Config Bits programados.
;
; O Device-ID podera ser:
;   01.0000.010 --> PIC16F627A
;   01.0000.011 --> PIC16F628A
;   01.0001.000 --> PIC16F648A
;
; ------------------------------------------------------------------------------------
; A memoria de configuracao fica no endereco 0x2000 a 0x2007 e tem o seguinte layout:
;                  UUuu:UUuu:UUuu:UUuu:RRrr:RRrr:DDdd:WWww
; onde somente os bytes UUuu e WWww podem ser gravados.
;
; A sequencia de comandos para gravar a memoria de configuracao deve ser a seguinte:
;
; 1 - Entrar em modo de programacao.
;
; 2 - Enviar o comando LOAD CONFIGURATION DATA para posicionar no endereco 0x2000
;     e com os dados zerados.
;
; 3 - Gravar os 4 bytes do User ID:
;     3.1 - Enviar o comando LOAD PROGRAM DATA com o dado do User ID.
;     3.2 - Enviar o comando BEGIN PROGRAMMING para iniciar a programacao
;           e aguardar o tempo para conclusao (TPROG).
;     3.3 - Enviar o comando INCREMENT ADDRESS para avancar para o proximo byte
;           na memoria de configuracao.
;     3.4 - Executar a partir do item 3.1 ate terminar o User ID.
;
; 4 - Gravar a Configuration Word.
;     4.1 - Na gravacao do ultimo byte do User ID o Program Counter ficou apontando 
;           para o primeiro byte reservado no endereco 0x2004. Pular o segundo byte 
;           reservado e o Device ID afendo apontar para a Configuration Word executando
;           3 vezes o comando INCREMENT ADDRESS. 
;     4.2 - Enviar o comando LOAD PROGRAM DATA para especificar o valor do dado.
;     4.3 - Converter o valor para 14 bits.
;     4.4 - Enviar o valor.
;     4.5 - Enviar o comando BEGIN PROGRAMMING para iniciar a programacao
;           e aguardar o tempo para conclusao (TPROG).   
;
; 5 - Sair do modo de programacao.
;
; ------------------------------------------------------------------------------------
; Entrada: O buffer iniciado em 'dataLen' deve conter os dados segundo o layout:
;          UUuu:UUuu:UUuu:UUuu:RRrr:RRrr:DDdd:WWww. Os bytes RRrr nao serao gravados.
;
; Saida  : Memoria de configuracao gravada.
;
; ------------------------------------------------------------------------------------
confWrite      call    picStop
               movlw   .250
               call    delay_ms
               call    picStart

               ; Envia comando LOAD CONFIGURATION DATA para posicionara no
               ; endereco 0x2000 (memoria de configuracao.
               ; Os dados devem ser zeros (0x0000) 
               movlw   LOAD_CFG                  ; LOAD CONFIGURATION 
               movwf   pgmr_cmd                  ; carrega comando
               call    picSendCmd                ; envia o comando
               clrf    pgmr_data1                ; MSB data =0x00
               clrf    pgmr_data0                ; LSB data =0x00
               call    picSendData               ; envia dado inicial

               ; Grava User ID no endereco 0x2000
               movlw   .4                        ; numero de bytes do User ID
               movwf   R14                       ; carrega o contador
confWrite2     movlw   LOAD_DPM                  ; LOAD PROGRAM DATA
               movwf   pgmr_cmd                  ; carrega registrador
               call    picSendCmd                ; envia o comando
               movlw   0x3F                      ; MSB do User ID
               movwf   pgmr_data1                ; carrega registrador
               movlw   0xFF                      ; LSB do User ID
               movwf   pgmr_data0                ; carrega registrador
               call    picConvert14                 ; converte  a 14 bits
               call    picSendData               ; 16 bits para completar o comando
               movlw   BPOC                      ; BEGIN PROGRAMMING
               movwf   pgmr_cmd                  ; carrega registrador
               call    picSendCmd                ; inicia a programacao
               call    TPROG                     ; tempo regulamentar
               movlw   INC_ADDR                  ; INCREMENT ADDRESS
               movwf   pgmr_cmd                  ; carrega registrador 
               call    picSendCmd                ; avanca um byte na memoria 
               decfsz  R14, f                    ; fim dos bytes do User ID???
               goto    confWrite2                ; NAO --> continua gravando 

               ; Aponta Config Word no endereco 0x2007
               movlw   .3                        ; numero de bytes a pular
               movwf   R14                       ; carrega o contador
confWrite3     movlw   INC_ADDR                  ; INCREMENT ADDRESS
               movwf   pgmr_cmd                  ; carrega o comando
               call    picSendCmd                ; envia o comando 
               decfsz  R14, f                    ; terminou???
               goto    confWrite3                ; NAO -->

               ; Grava Configuration Word
               movlw   LOAD_DPM                  ; LOAD PROGRAM DATA
               movwf   pgmr_cmd                  ; carrega o comando
               call    picSendCmd                ; envia o comando

               movf    dataBytes+.0, w           ; pega LSB do Congif Word no buffer
               movwf   pgmr_data0                ; carrega LSB  
               movf    dataBytes+.1, w           ; pega MSB do Congif Word no buffer
               movwf   pgmr_data1                ; carrega MSB

               call    picConvert14              ; converte a 14 bits

               call    picSendData               ; envia o valor

               movlw   BPOC                      ; BEGIN PROGRAMMING
               movwf   pgmr_cmd                  ; carrega o comando
               call    picSendCmd                ; envia o comando
               call    TPROG                     ; tempo para executar

               call    picStop                   ; sai do modo de programacao
               return                            ; fim

