; ---------------------------------------------------------------------------------------------
; picprog.inc                                                                        08/08/2020
;
; FUNCOES PARA PROGRAMACAO DE MICROCONTROLADORES PIC
;
; REVISOES:
; 08/08/2020 - Versao inicial
;
; ---------------------------------------------------------------------------------------------
; MECANISMO DE ACESSO AS MEMORIAS:
;
; Existem 3 tipos de memorias na familia 16F:
;
; Memoria de programa
;
;   Essa memoria e' implementada dos enderecos 0x0000 a 0x3FFF e esta dividida e 2 regioes:
;
;     A regiao do usuario que vai de 0x0000 a 0x1FFF.
;        A regiao do usuario so pode ser acessada quando o chip entra em modo de programacao.
;        Ao entrar nesse modo o PC (Program Counter) apontar para a regiao do usuario.
;
;     A regiao de configuracao que vai de 0x2000 a 0x3FFF (o maximo depende do chip).
;        A regiao de configuracao pode ser acessada usando-se o comando LOAD-CONFIGURATION que
;        carrega o PC (Program Counter) com o valor 0x2000.
;
;
; Memoria EEPROM
;
;
; ---------------------------------------------------------------------------------------------
; MODO DE PROGRAMACAO (Alta voltagem)
; Para entrar em modo de programacao de alta voltagem:
; 1 - Manter PGC = 0 
; 2 - Manter PGD = 0
; 3 - MCLR/Vpp = 13V - esperar no minimo 5us
; 4 - Vdd = 5v - esperar no minimo 5us
; 5 - Manter PGM=0 (Isso nao se verificou. Esse pino ficou flutuante e funcionou)
;
; Para acessar a memoria de Programa (regiao do usuario (0x0000 a 0x1FFF):
; 1 - Entrar em modo de programacao (ver acima)
; 
;
;
; ---------------------------------------------------------------------------------------------
; REQUISITOS:
;
; Estas funcoes requerem a definicao dos seguintes simbolos no programa:
;pgmr_flags   RES     1      ; flags do servico de programacao
;pgmrmode     equ     0      ; 0 -> prog mode Off       1 -> prog mode On
;
;pgmr_rg1     RES     1      ; registrador geral 1
;pgmr_rg2     RES     1      ; registrador geral 1
;pgmr_cmd     RES     1      ; command buffer
;pgmr_data1   RES     1      ; MSB data buffer
;pgmr_data0   RES     1      ; LSB data buffer
;pgmr_bits    RES     1      ; contador de bits
;
;#define     PGMR_CNTL    porta    ; Porta de controle onde sera definido o bit de clock
;#define     PGMR_DATA    porta    ; Porta de dados onde sera definido o bit de dados
;#define     PGMR_IO      reg      ; Registrador de controle das portas de I/O
;#define     PGMR_PWHI    porta    ; Porta onde serao definidos o bit de controle da alta voltagem (13V) - P_HI
;#define     PGMR_PWLO    porta    ; Porta onde serao definidos o bit de controle da tensao de alimentacao (5V) - P_LO
;P_CK        equ          n        ; bit de saida do clock
;P_RD        equ          n        ; bit de entrada de dado (READ)
;P_WD        equ          n        ; bit de saida de dado   (WRITE)
;P_HI        equ          n        ; bit de controle da alta voltagem
;P_LO        equ          n        ; bit de controle da tensao de alimentacao
; ---------------------------------------------------------------------------------------------
; RELACAO DE FUNCOES:
; picStart.............. PREPARA O CIRCUITO INTEGRADO PARA SER PROGRAMADO
; picStop............... DESLIGA O CIRCUITO INTEGRADO
; picEraseProg.......... APAGA A MEMORIA DE PROGRAMA
; picEraseEprom......... APAGA A MEMORIA EEPROM
; picReceive............ LE 16 BITS DA MEMORIA DE PGM OU DA MEMORIA DE DADOS
; picSendData........... ENVIA 16 BITS PARA A MEMORIA DE PGM OU DA MEMORIA DE DADOS
; picSendCmd............ ENVIA 6 BITS DE COMANDO
; picConvert14.......... TRANSFORMA VALOR EM FORMATO DE DADOS 14 BITS PARA GRAVAR/LER O CI 
; picEpBits............. FORMATA OS 16 BITS DE DADOS COM O VALOR DE 8 BITS PARA GRAVACAO NA EEPROM
; picConvert8........... TRANSFORMA 14 BITS NO FORMATO NORMAL 8 + 8 BITS
; ---------------------------------------------------------------------------------------------


; ---------------------------------------------------------------------------------------------
; Entra em modo de programacao                                                       08/08/2020
;
; Entrada: nao ha
; Saida  : Tensao de alimentacao ligada
;          Tensao de programacao ligada
;          Pino clock abaixado
;          Pino data abaixado
;
; Notas:
; 1 - O pino P_HI e' Active-low
; 2 - O pino P_LO e' Active-Low
;
; ---------------------------------------------------------------------------------------------
picStart       call    pwmOn                     ; ativa a bomba de tensao
               movlw   .50
               call    delay_ms
               call    pwmOff                    ; desativa a bomba de tensao
               movlw   .50
               call    delay_ms

               bcf     PGMR_CNTL,P_CK            ; clock=0
               bcf     PGMR_DATA,P_WD            ; data=0
               nop                               ; tempo
               bsf     STATUS, RP0               ; banco 1
               bcf     PGMR_IO, P_WD             ; pino data ----> saida
               bcf     PGMR_IO, P_CK             ; pino clock ---> saida
               bcf     PGMR_IO, P_HI             ; pino HiVolt --> saida
               bcf     PGMR_IO, P_LO             ; pino LoVolt --> saida
               bcf     STATUS, RP0               ; banco 0
               movlw   .250                      ; us 
               call    delay_us                  ; espera estabilizar 
               call    pwmOn                     ; ativa a bomba de tensao
               movlw   .220
               call    delay_us
               bcf     PGMR_PWHI,P_HI            ; liga a alta voltagem (13V)
               movlw   .250                      ; minimo recomendado pelo data sheet --> 5us 
               call    delay_us                  ; espera estabilizar 
               bcf     PGMR_PWLO,P_LO            ; liga a alimentacao do CI (5v)
               movlw   .250                      ; minimo recomendado pelo data sheet --> 5us 
               call    delay_us                  ; espera estabilizar 
               bsf     pgmr_flags, pgmrmode      ; indica que esta em modo de programacao
               return                            ; fim
; ---------------------------------------------------------------------------------------------
; Sai do modo de programacao                                                         08/08/2020 
;
; Entrada: nao ha
; Saida  : Tensao de alimentacao desligada
;          Tensao de programacao desligada
;          Pino clock abaixado
;          Pino data abaixado
;
; Notas:
; 1 - O pino P_HI e' Active-low
; 2 - O pino P_LO e' Active-Low
;
; ---------------------------------------------------------------------------------------------
picStop        bsf     PGMR_PWLO,P_LO            ; corta a alimentacao do CI (5v)
               bsf     PGMR_PWHI,P_HI            ; corta a alta voltagem (13V)
               call    pwmOff
               bcf     PGMR_CNTL,P_CK            ; clock = 0
               bcf     PGMR_DATA,P_WD            ; data = 0
               bsf     STATUS, RP0               ; banco 1
               bcf     PGMR_IO, P_WD             ; pino data ----> saida
               bcf     PGMR_IO, P_CK             ; pino clock ---> saida
               bcf     PGMR_IO, P_HI             ; pino HiVolt --> saida
               bcf     PGMR_IO, P_LO             ; pino LoVolt --> saida
               bcf     STATUS, RP0               ; banco 0
               movlw   .250                      ; minimo recomendado pelo data sheet --> 5us 
               call    delay_us                  ; espera estabilizar 
               bcf     pgmr_flags, pgmrmode      ; indica que saiu do modo de programacao
               return

; ---------------------------------------------------------------------------------------------
; Apaga a memoria de configuracao                                                    05/08/2020
;
; Entrada: Nao ha
; Saida  : Memoria de configuracao apagada
;          Clock --> PGMR_CNTL<P_CK> = 0 
;
; ---------------------------------------------------------------------------------------------
picEraseConfig call    picStart
               movlw   LOAD_CFG                  ; comando LOAD CONFIGURATION
               movwf   pgmr_cmd                  ; guarda comando no buffer
               call    picSendCmd                ; envia comando
               call    picEraseMem               ; envia dado 
               call    TERA                      ; tempo para apagar
               call    picStop
               return

; ---------------------------------------------------------------------------------------------
; Apaga a memoria de programa                                                        28/07/2020
;
; Entrada: Nao ha
; Saida  : Memoria de programa apagada
;          Clock --> PGMR_CNTL<P_CK> = 0 
; ---------------------------------------------------------------------------------------------
picEraseProg   call    picStart                  ; modo de programacao
               movlw   LOAD_DPM                  ; comando LOAD DATA FOR PROGRAM MEMORY
               movwf   pgmr_cmd                  ; guarda comando no buffer
               call    picSendCmd                ; executa o comando LOAD DATA FOR PGM MEMORY
picEraseMem    movlw   0x3F
               movwf   pgmr_data1                ; limpa
               movlw   0xFF 
               movwf   pgmr_data0                ; limpa
               call    picConvert14              ; converte a 14 bits
               call    picSendData               ; envia dados ficticios
               movlw   BULK_EPM                  ; comando BULK ERASE PGM MEMORY
               movwf   pgmr_cmd                  ; guarda comando no buffer
               call    picSendCmd                ; executa o comando BULK ERASE PGM MEMORY
               call    TERA                      ; tempo para executar
               call    picStop                   ; sai do modo programacao
               return                            ; fim do comando

; ---------------------------------------------------------------------------------------------
; Apaga a memoria EEPROM                                                             08/08/2020
;
; Entrada: Nao ha
; Saida  : EEPROM totalmente apagada
; ---------------------------------------------------------------------------------------------
picEraseEprom  call    picStart
             ;  movlw   LOAD_DDM
             ;  movwf   pgmr_cmd
             ;  call    picSendCmd
             ;  movlw   0x3F
             ;  movwf   pgmr_data1
             ;  movlw   0xFF
             ;  movwf   pgmr_data0 
             ;  call    picConvert14
             ;  call    picSendData
               movlw   BULK_EDM                  ; comando BULK ERASE DATA MEMORY
               movwf   pgmr_cmd                  ; guarda comando no buffer
               call    picSendCmd                ; executa o comando BULK ERASE PGM MEMORY
               call    TERA                      ; tempo para executar
               call    picStop
               return


; ---------------------------------------------------------------------------------------------
; ENVIA 6 BITS DE COMANDO                                                            28/07/2020
;
; Entrada: O registrador 'pgmr_cmd' contem o comando 
; Saida  : Comando enviado.
;          Clock --> PGMR_CNTL<P_CK> = 0 
; ---------------------------------------------------------------------------------------------
picSendCmd     movlw   .6                        ; 6 bits de comando
               movwf   pgmr_bits                 ; guarda no contador de bits
picSendCmd1    bcf     PGMR_DATA,P_WD            ; desliga o bit de dado
               rrf     pgmr_cmd,f                ; LSB para o Carry bit
               btfss   STATUS,C                  ; testa o Carry
               goto    picSendCmd2               ; se carry = 0, pula uma instrucao
               bsf     PGMR_DATA,P_WD            ; liga o bit de dado
picSendCmd2    call    picStrobe                 ; envia
               decfsz  pgmr_bits,f               ; contabiliza bits transferidos
               goto    picSendCmd1               ; se nao chegou a zero, continua...
               return                            ; senao, termina

; ---------------------------------------------------------------------------------------------
; ENVIA 16 BITS DE DADOS PARA A MEMORIA DE PROGRAMA OU DA EEPROM.                    28/07/2020
;
; Se o comando LOAD DATA FOR PROGRAM MEMORY preceder este comando, o dado sera enviado 
; para a memoria de programa.
;
; Se o comando LOAD DATA FOR DATA MEMORY preceder este comando, o dado sera enviado
; para a memoria de dados (EEPROM).
;
; Os bits serao enviados comecando plo LSB (pgmr_data0<0>).
;
; Entrada: pgmr_data1:pgmr_data0 contem o dado a enviar.
; Saida  : dado enviado
;          Clock --> PGMR_CNTL<P_CK> = 0 
;
; ---------------------------------------------------------------------------------------------
picSendData    movlw   .16                       ; 16 bits de dados
               movwf   pgmr_bits                 ; carrega contador de bits
picSendData1   bcf     PGMR_DATA,P_WD            ; desliga o bit de dado
               rrf     pgmr_data1,f              ; pgmr_data1(LSB) ---> carry
               rrf     pgmr_data0,f              ; Carry ---> pgmr_data0(MSB) pgmr_data0(LSB) ---> carry 
               btfss   STATUS,C                  ; testa o Carry
               goto    picSendData2              ; se carry = 0, pula uma instrucao
               bsf     PGMR_DATA,P_WD            ; liga o bit de dado
picSendData2   call    picStrobe                 ; envia
               decfsz  pgmr_bits,f               ; fim dos dados???
               goto    picSendData1              ; NAO --> continua...
               return                            ; 

; ---------------------------------------------------------------------------------------------
; LEITURA DE UM BYTE DE 16 BITS DA MEMORIA DE PROGRAMA.                              28/07/2020
;
; Le a memoria de programa e devolve o resultado no par de registradores pgmr_data1:pgmr_data0.
; O bit de dado PGMR_IO<P_RD> sera  modificado para 'entrada' durante a leitura e voltara
; para 'saida' ao final.
;
; Entrada: O PC deve apontar para a localizacao de memoria a ser lida.
; Saida  : O par pgmr_data1:pgmr_data0 contera o resultado.
;          Clock --> PGMR_CNTL<P_CK> = 0 
; ---------------------------------------------------------------------------------------------
picGetByte     bsf     STATUS,RP0                ; banco 1
               bsf     PGMR_IO,P_RD              ; muda bit para entrada
               bcf     STATUS,RP0                ; banco 0
               movlw   .16                       ; 16 bits de dados
               movwf   pgmr_bits                 ; carrega contador de bits   
picReceive1    call    picStrobe                 ; recebe
               movf    PGMR_DATA,w               ; le a porta
               movwf   pgmr_rg1                  ; salva o conteudo lido
               movlw   P_RD                      ; pega o numero do bit lido na porta
               addlw   .1                        ; soma 1 nele
               movwf   pgmr_rg2                  ; salva o numero de deslocamentos
               rrf     pgmr_rg1,f                ; empurra o bit lido para o Carry
               decfsz  pgmr_rg2,f                ; empurra o bit lido para o Carry
               goto    $-2                       ; empurra o bit lido para o Carry
               rrf     pgmr_data1,f              ; injeta o carry no MSB do pgrm_data1 --> C=LSB
               rrf     pgmr_data0,f              ; injeta LSB de pgmr_data1 no MSB do pgrm_data0
               decfsz  pgmr_bits,f               ; tira 1 do numero de bits transferidos
               goto    picReceive1              ; se nao acabou, continua lendo, senao...
               bsf     STATUS,RP0                ; banco 1
               bcf     PGMR_IO,P_RD              ; muda bit para saida
               bcf     STATUS,RP0                ; banco 0
               return

; ---------------------------------------------------------------------------------------------
; Leitura bufferizada da memoria de configuracao                                     08/08/2020 
;
; Le totalmente a memoria de configuracao armazenando os dados num buffer cujo endereco esta
; especificado pelo registrador W.
; 
; Entrada: W contem o endereco do buffer de leitura.
; Saida  : Buffer contem os dados da memoria de configuracao.
;
; ---------------------------------------------------------------------------------------------
picReadConfig  movwf   FSR                       ; carrega indexador com endereco do buffer
               call    picStart                  ; entra em programacao
               movlw   LOAD_CFG                  ; comando LOAD-CONFIGURATION no endereco 0x2000
               movwf   pgmr_cmd                  ; armazena no buffer
               call    picSendCmd                ; envia o comando
               movlw   0xFF                      ; falso config
               movwf   pgmr_data1                ; armazena no buffer
               movlw   0xFF                      ; falso config
               movwf   pgmr_data0                ; armazena no buffer
               call    picConvert14              ; formata 14 bits em 16
               call    picSendData               ; envia dado
               movlw   .8                        ; 8 bytes a ler
               movwf   R11                       ; carrega contador
picReadConf2   call    picReadPgm                ; le um byte na memoria de configuracao
               movf    pgmr_data0, w             ; na memoria e' invertido
               movwf   INDF
               incf    FSR, f
               movf    pgmr_data1, w             ; na memoria e' invertido
               movwf   INDF
               incf    FSR, f
               call    picIncAddr
               decfsz  R11, f
               goto    picReadConf2
               call    picStop
               return

; ---------------------------------------------------------------------------------------------
; Leitura de um byte na memoria de programa                                          08/08/2020 
;
; Le a memoria de configuracao ou de programa, dependendo de onde o PC esteja apontando.
;
; Entrada: PC aponta para a localizacao de memoria a ser lida (programa/configuracao)
; Saida  : O par pgmr_data1:pgmr_data0 contem o resultado.
; ---------------------------------------------------------------------------------------------
picReadPgm     movlw   READ_DPM                  ; comando READ-DATA-FROM-PROGRAM-MEMORY
               movwf   pgmr_cmd                  ; coloca no buffer
               call    picSendCmd                ; envia o comando
               call    picGetByte                ; leitura
               call    picConvert8               ; converte a um formato civilizado
               return

; ---------------------------------------------------------------------------------------------
; Incrementa o Program Counter                                                       08/08/2020
;
; Entrada: Nao ha
; Saida  : PC incrementado de um byte.
; ---------------------------------------------------------------------------------------------
picIncAddr     movlw   INC_ADDR                  ; comando INCREMEMT ADDRESS
               movwf   pgmr_cmd                  ; armazena no buffer
               call    picSendCmd                ; envia o comando
               return

; ---------------------------------------------------------------------------------------------
; LEVANTA E ABAIXA O SINAL DE CLOCK                                                  30/07/2020
;
; Esta funcao levanta e abaixa o sinal de clock para enviar/receber um comando ou dado 
; ao microconctrolador que esta sendo programado. 
; Apos ser levantado, o sinal sera mantido por 1 ciclo de instrucao antes de ser abaixado.
; Todo o processo, desde o CALL desta funcao, usara 7 ciclos.
;
; Entrada: Nao ha
; Saida  : Clock --> PGMR_CNTL<P_CK> = 0 
; --------------------------------------------------------------------------------------------
picStrobe      bsf     PGMR_CNTL,P_CK            ; levanta o sinal de clock
               nop                               ; mantem estabilidade do dado
               bcf     PGMR_CNTL,P_CK            ; abaixa o sinal de clock para enviar
               return

; ---------------------------------------------------------------------------------------------
; COVERTE VALORES PARA 14 BITS                                                       28/07/2020
;
; Entrada: pgrm_data1:pgmr_data0 contem o valor a transformar (MAX 0x3FFF)
; Saida  : pgrm_data1:pgrm_data0 convertido para 0MMMMMML:LLLLLLL0
; ---------------------------------------------------------------------------------------------
picConvert14   rlf    pgmr_data0, f                ; data0 = 1111.111x   C=0
               rlf    pgmr_data1, f                ; data1 = 0111.1110   C=0
               bcf    pgmr_data1,7                 ; data1 = 0111.1110
               bcf    pgmr_data0,0                 ; data0 = 1111.1110   ---> RESULTA=0.11/1111/0111/1111.0
               return


; ---------------------------------------------------------------------------------------------
; CONVERTE VALORES 14 BITS PARA 16 BITS                                              28/07/2020 
;
; Converte valores em formato 14 bits lidos na memoria de programa para formato
; civilizado de 16 bits.
;
; Entrada: pgrm_data1:pgrm_data0 no formato 0MMMMMML:LLLLLLL0
; Saida  : pgrm_data1:pgmr_data0 no formato 00MMMMMM:LLLLLLLL (MAX 0x3FFF)
; ---------------------------------------------------------------------------------------------
picConvert8    rrf    pgmr_data1,f
               rrf    pgmr_data0,f
               movlw  0x3F
               andwf  pgmr_data1, f
               return

; ---------------------------------------------------------------------------
; Grava um byte na memoria de programa                             09/08/2020
;
; Requisitos: O chip deve estar em modo de programacao.
;
; Entrada: O par pgmr_data1:pgmr_data0 deve conter o valor no formato MSB:LSB.
; Saida  : Byte gravado na memoria de programa.
; ----------------------------------------------------------------------------
picWriteData   call    picConvert14              ; formata em 14 bits antes de enviar
               movlw   LOAD_DPM                  ; LOAD DATA FOR PROGRAM MEMORY
               movwf   pgmr_cmd                  ; carrega buffer de comando
               call    picSendCmd                ; envia o comando
               call    picSendData               ; envia o dado
               movlw   BPOC                      ; BEGIN PROGRAM CYCLE
               movwf   pgmr_cmd                  ; carrega comando
               call    picSendCmd                ; envia
               call    TPROG                     ; tempo requerido para BPOC
               return

; ---------------------------------------------------------------------------------------------
; Le um byte na EEPROM                                                               08/08/2020
;
; Entrada: O Program Counter aponta para o endereco a ser lido na EEPROM.
; Saida  : W contem o resultado da leitura.
;          O Program Counter aponta para o proximo endereco na EEPROM.
; ---------------------------------------------------------------------------------------------
picGetEpByte   movlw   READ_DDM 
               movwf   pgmr_cmd 
               call    picSendCmd                ; envia o comando
               call    picGetByte                ; leitura --> pgmr_data1:pgmr_data0
               call    picConvert8               ; converte pgmr_data1:pgmr_data0 a 8 bits
               call    picIncAddr                ; aponta o proximo endereco na EEPROM
               movf    pgmr_data0, w             ; carrega W com o dado lido
               return
 
; ---------------------------------------------------------------------------
; Grava um byte na memoria EEPROM                                  10/08/2020
;
; Entrada: O par [pgmr_data1:pgmr_data0] MSB:LSB contem o valor a ser gravado.
;          O Program Counter aponta para o endereco da EEPROM.
;
; Saida  : Byte gravado na EEPROM.
;          O Program Counter incrementado de 1.
;
; ---------------------------------------------------------------------------
picPutEpByte   movlw   LOAD_DDM                  ; LOAD DATA FOR DATA MEMORY
               movwf   pgmr_cmd                  ; carrega o comando
               call    picSendCmd                ; envia
               call    picConvert14              ; pgmr_data1:pgmr_data0 para 16 bytes
               call    picSendData               ; envia pgmr_data1:pgmr_data0
               movlw   BPOC                      ; BEGINM PROGRAM ONLY CYCLE
               movwf   pgmr_cmd                  ; carrega o comando
               call    picSendCmd                ; envia
               call    TDPROG                    ; tempo requerido
               call    picIncAddr                ; incrementa Program Counter
               return                            ; fim

; ---------------------------------------------------------------------------------------------
; ---------------------------------------------------------------------------------------------
; ---------------------------------------------------------------------------------------------
TDPROG       movlw  .5            ; maximo recomendado pelo data sheet --> 6ms
             goto   TPROGX

TERA         movlw  .5            ; maximo recomendado pelo data sheet --> 6ms
             goto   TPROGX

TPROG        movlw  .8            ; maximo recomendado pelo data sheet --> 2,5ms
                                  ; mas nao funciona - deixa com 8ms
TPROGX       movwf  R3
             rlf    R3, f         ; multiplica por 2
             rlf    R3, f         ; multiplica por 2 novamente
TPROG2       movlw  .250          ;
             call   delay_us
             decfsz R3, f
             goto   TPROG2
             return

