; ---------------------------------------------------------------------------------------------
; Servicos para Porta Serial atraves de Interrupcao de Rx                            28/11/2019
;
; STATUS: FUNCIONA
;
;---------------------------------------------------------------------------------------------
; Este modulo fornece macro instrucoes para operacoes de I/O buferizadas usando a Porta Serial
; do microcontrolador (USART).
;
; As portas seriais, do microcontrolador e do dispositivo externo, devem ser configuradas com 
; Baud Rate iguais a partir de 9600BPS, inclusive.
;
; Ao ler um dispositivo serial externo, como um teclado de terminal, os dados serao acumulados
; num buffer da memoria RAM ate que seja detetado um caracter delimitador de fim de linha
; enviado pela tecla ENTER (0x0D) do terminal. O buffer de leitura contera, inclusive, o 
; caracter delimitador recebido do dispositivo externo.
;
; Este modulo foi testado no PIC16F628A e PIC18F4520 com sucesso e funcionou com:
;
;  * Portas Seriais (COMx) do PC no Windows/Xp usando o Hyperterminal.  
;
;  * Portas USB-Seriais no PC com Linux usando o programa PICOCOM com as seguintes opcoes:
;    picocom /dev/ttyUSB0 -b 9600 -c
;
;  * Dispositivo Bluetooth HC-06 recebendo dados de um aplicativo Smartphone dispositivo.
;
; ---------------------------------------------------------------------------------------------
; MODO DE USAR:
;
; Os servicos deste modulo sao distribuidos atraves das macro instrucoes:
;
;  SERIAL_CLEAR ............................ Limpa a tela de um terminal
;  SERIAL_FUNCTIONS ........................ Gera as funcoes para o servico da porta serial
;  SERIAL_INTERRUPT ........................ Desvia para a interrupcao (ISR) da porta serial
;  SERIAL_NL ............................... Avanca para nova linha no terminal
;  SERIAL_OPEN fosc, baud, dlm ............. Abre a porta serial
;  SERIAL_PUT 'char' ....................... Envia um caracter pela porta serial
;  SERIAL_READ buffer ...................... Le a porta serial
;  SERIAL_WRITE 'string' ................... Envia uma 'string' (final 0x0D) pela porta serial
;
; Para usar os servicos deste modulo, seguir as instrucoes abaixo:
;
; 1 - Incluir no inicio do programa, antes da RAM, o arquivo 'serialport_services.inc' que
;     fornecera as macros e funcoes necessarias.
;
; 2 - Inserir na RAM do programa os registradores abaixo:
;SERDLM   res       .1  ; delimitador de strings
;SERWR    res       .1  ; Salvamento de contexto do registrador W
;SERST    res       .1  ; Salvamento de contexto do registrador STATUS
;SERFSR   res       .1  ; Salvamento de indexador
;SERFLAGS res       .1  ; FLAGS DO SERVICO 
;SEREOT   equ       .0  ;   1 --> fim de transmissao
;
; 3 - Definir na RAM, em qualquer banco, um buffer de tamanho suficiente para conter os dados.
;     Por exemplo:
;
;     mySerBuff   equ   0xA0    ; buffer no banco 2
;
; 4 - Na sessao MAIN do programa usar a macro SERIAL_FUNCTIONS para definir o servicos:
;
;     MAIN   CODE
;            SERIAL_FUNCTIONS 
; 
; 5 - Na logica principal do programa usar a macro SERIAL_OPEN para abrir o servico:
;
;            SERIAL_OPEN .20, .9600, 0x0D
;
;     onde: .20 .................... Frequencia de clock. (.4 ou .20 por enquanto...)
;           .9600 .................. Baud Rate desejado em BPS.
;           0x0D ................... Caracter delimitador de string (terminais Linux;Windows).
;                                    Usar 0x0A quando recebendo mensagens de aplicativos Bluetooth.
;
; 6 - Apos aberto, pode-se ler o dispositivo serial:
;    
;           SERIAL_READ  buffer
;
;     onde: buffer ............. e' o buffer de leitura (qualquer banco da RAM)
;      
; 7 - No local onde inicia as rotinas de interrupcao (ISR) usar a macro SERIAL_INTERRUPT para
;     chamar a ISR da Serial. Exemplo:
;
;     RESTV   CODE    0x000 
;             goto    inicio
;     INTV    CODE    0x004 
;             goto    FLIH
;              .
;              .
;              .
;     FLIH     btfsc  ...         ; testa mascara de interrupcao 1
;              goto   isr_1       ; executa ISR da interrupcap 1
;              btfsc  ...         ; testa mascara de interrupcao 2
;              goto   isr_2       ; executa ISR da interrupcap 2
;
;              SERIAL_INTERRUPT   ; remete para a ISR da Serial
;
;              btfsc  ...         ; testa mascara de interrupcao n
;              goto   isr_n       ; executa ISR da interrupcap n
;
;              retfie
;
; ---------------------------------------------------------------------------------------------


    ifndef __FAMILIAS_SYMBOL
      #include "..\..\Includes\familias.inc"
      messg "SERIAL_PORT_SERVIVES - Arquivo familias.inc foi incluido"
    endif

    ifndef SERIAL_PORT_SERVICES
#define  SERIAL_PORT_SERVICES  1
    messg "SERIAL_PORT_SERVICES - Arquivo seriaport_services.inc foi incluido"
    endif


; ---------------------------------------------------------------------------------------------
; SERIAL_OPEN - Configuracao da Porta Serial                                         24/11/2019
;
; fosc .................. Frequencia de clock. (4MHZ, 10MHZ, 16MHZ e 20MHZ)
; baud .................. Baud Rate desejado (BPS)
; dlm ................... Caracter delimitador de string.
;                         Aplicativo Bluetooth .......... 0x0A 
;                         Terminais Linux/Windows ....... 0x0D
;
; ---------------------------------------------------------------------------------------------
SERIAL_OPEN MACRO    fosc, baud, dlm

           movlw   dlm                    ; pega o delimitador de string
           movwf   SERDLM                 ; salva no bloco de controle

   ; --------------------------------- 
   ; PIC16F - Configura a porta serial 
   ; --------------------------------- 
   if __FAMILIA==16
    local kilo, baudrate
kilo  set  baud * .16
baudrate set fosc*.1000000
baudrate set baudrate - kilo
baudrate set (baudrate / kilo)
           movlw   baudrate               ; pega valor para gerar Baud Rate
           bsf     STATUS, RP0            ; banco 1
           movwf   SPBRG                  ; SET BAUD RATE GENERATOR
           bcf     STATUS, RP0            ; banco 0
           call    serialConfig           ; configura a porta serial
   endif

   ; --------------------------------- 
   ; PIC18F - Configura a porta serial 
   ; --------------------------------- 
   if __FAMILIA==18
    local kilo, baudrate
kilo  set  baud * .4
baudrate set fosc*.1000000
baudrate set baudrate - kilo
baudrate set (baudrate / kilo)
           movlw   high(baudrate)
           movwf   SPBRGH
           movlw   low(baudrate)
           movwf   SPBRG
           call    serialConfig           ; configura a porta serial
   endif
           ENDM

; ---------------------------------------------------------------------------------------------
; SERIAL_INTERRUPT - Define destino para a interrupcao do SERIAL                     24/11/2019
;
; Esta macro deve ser usada para desviar para arotina ISR da porta serial quando
; ocorrer uma interrupcao.
; ---------------------------------------------------------------------------------------------
SERIAL_INTERRUPT   MACRO
           btfsc  PIR1, RCIF
           goto   serPortISR
           ENDM

; ---------------------------------------------------------------------------------------------
; SERIAL_READ - Leitura do SERIAL                                                    24/11/2019
;
; Entrada: W contem o endereco inicial do buffer de leitura.
; ---------------------------------------------------------------------------------------------
SERIAL_READ  MACRO  buffadd
   ifndef _numq
     variable _numq
_numq set 0
   endif
_numq set _numq + 1

   if __FAMILIA==16
           movlw   buffadd               ; endereco do inicio do buffer
           movwf   FSR                   ; carrega indexador
   else
           lfsr    FSR1, buffadd
   endif

           ; Habilita a interrupcao Rx
           bcf     SERFLAGS, SEREOT      ; limpa indicador de fim de transmissao 
           bcf     PIR1, RCIF            ; limpa flag da interrupcao
   if __FAMILIA==16
           bsf     STATUS, RP0           ; banco 1
   endif
           bsf     PIE1, RCIE            ; habilita interrupcao Rx
   if __FAMILIA==16
           bcf     STATUS, RP0           ; banco 0   
   endif

           ; Espera chegar mensagem da Porta Serial   
serd#v(_numq)    btfss   SERFLAGS, SEREOT          ; chegou mensagem da serial?
           goto    serd#v(_numq)                   ; NAO --> espera chegar mensagem

           ; Desabilita a interrupcao Rx - 05/06/2020
           bcf     SERFLAGS, SEREOT      ; limpa indicador de fim de transmissao 
           bcf     PIR1, RCIF            ; limpa flag da interrupcao
   if __FAMILIA==16
           bsf     STATUS, RP0           ; banco 1
   endif
           bcf     PIE1, RCIE            ; desabilita interrupcao Rx
   if __FAMILIA==16
           bcf     STATUS, RP0           ; banco 0   
   endif

           ENDM



; ---------------------------------------------------------------------------------------------
; SERIAL_NL - Posiciona na proxima linha do terminal                                 27/11/2019  
; ---------------------------------------------------------------------------------------------
SERIAL_NL  MACRO
           call  serialNewLine
           ENDM


; ---------------------------------------------------------------------------------------------
; SERIAL_CLEAR - Limpa o terminal                                                    27/11/2019
; ---------------------------------------------------------------------------------------------
SERIAL_CLEAR MACRO
           call    serialClear
           ENDM

; ---------------------------------------------------------------------------------------------
; SERIAL_PUT - Envia um caracter ao terminal serial                                  27/11/2019
; ---------------------------------------------------------------------------------------------
SERIAL_PUT MACRO   chr
           movlw   chr
           call    serialPut
           ENDM

; ---------------------------------------------------------------------------------------------
; SERIAL_WRITE - Envia uma string ao terminal serial                                 27/11/2019
;
; Entrada: FSR/FSR1 (dependendo da familia) deve apontar para o inicio da string
; ---------------------------------------------------------------------------------------------
SERIAL_WRITE MACRO str
 ifndef _numq
    variable _numq
_numq  set  0
 endif
_numq  set _numq + 1
  if __FAMILIA==16
           clrf    SERWR              ; usa como indexador da TABLE READ
srw16#v(_numq)
           movf    SERWR, w           ; carrega indice da TABLE READ
           incf    SERWR, f           ; avanca o indice
           call    str                ; busca um caracter na TABLE READ
           movwf   SERST              ; salva o caracter
           xorlw   0x0D               ; verifica o caracter 
           btfsc   STATUS, Z          ; fim da string?
           goto    srf#v(_numq)       ; SIM --> encerra
           movf    SERST, w           ; recupera o caracter  
           call    serialPut          ; envia o caracter
           goto    srw16#v(_numq)     ; proximo caracter
srf#v(_numq) 
  endif

  if __FAMILIA==18
           call    serialWrite
           dw      str,0x0D
  endif
             ENDM


; ---------------------------------------------------------------------------------------------
; DEFINICAO DAS FUNCOES DE I/O PARA PORTA SERIAL                                     24/11/2019
;
; SERWR .................. Registrador para salvar W.
; SERST .................. Registrador para salvar STATUS.
; SERFLAGS ............... Um registrador de flags do programa.
; SEREOT ................. Bit do registrador de flags para indicar fim de transmissao do SERIAL.
;                          Esse bit sera usado pela rotina ISR.
; ---------------------------------------------------------------------------------------------
SERIAL_FUNCTIONS MACRO

; ---------------------------------------------------------------------------------------------
; serialConfig - Configuracao da porta serial                                        28/11/2019
;
; Esta funcao e' usada pela macro SERIAL_OPEN para configurar a porta serial.
;
; Entrada: O Baud Rate je deve ter sido especificado na macro SERIAL_OPEN
; Saida  : Nao ha
;
; ---------------------------------------------------------------------------------------------
serialConfig
 if __FAMILIA==16
          bsf     STATUS, RP0            ; banco 1
           movlw   0x06                   ; TRISB<2:1> ==> Rx/Tx
           iorwf   TRISB , f              ; I/O para as portas da USART
           bsf     TXSTA,  BRGH           ; High Speed
           bcf     TXSTA,  SYNC           ; Async Mode 
          bcf     STATUS, RP0            ; banco 0
           bsf     RCSTA,  SPEN           ; Enable Serial Port
           bsf     RCSTA,  CREN           ; Enable Reception
           bsf     INTCON, PEIE           ; Enable Peripheric Interruptions
           bsf     INTCON, GIE            ; Enable General Interruptions
           return
 endif

 if __FAMILIA==18
           bcf     RCON, IPEN             ; sem prioridade de interrupcoes
           movlw   0xC0                   ; TRISC<7:6> ==> Rx(In)/Tx(Out)
           iorwf   TRISC, f               ; I/O para as portas da USART
           clrf    BAUDCON                ; limpa BAUDCON
           bsf     BAUDCON,BRG16          ; forca 16 bits em SPBRG
           clrf    RCSTA                  ; limpa receiver
	       bsf     RCSTA,SPEN             ; habilita porta serial
           bsf     RCSTA,CREN             ; habilita o receiver
           clrf    TXSTA                  ; limpa tramsmitter
           bcf     TXSTA,SYNC             ; modo assincrono
           bsf     TXSTA,TXEN             ; habilita o transmitter
           bsf     TXSTA,BRGH             ; estabelece alta velocidade
           bsf     INTCON,PEIE            ; habilita interrupcoes de perifericos
           bsf     INTCON,GIE             ; habilita interrupcoes globalmente
           movf    RCREG, w               ; descarrega o buffer da USART
           movf    RCREG, w               ; descarrega o buffer da USART
           movf    RCREG, w               ; descarrega o buffer da USART
           return                         ; fim da configuracao
  endif 


; ---------------------------------------------------------------------------------------------
; serialPut - Envia um caracter pela porta serial                                    27/11/2019
;
; Entrada: W contem o caracter a ser enviado
; Saida  : Nao ha
; ---------------------------------------------------------------------------------------------
serialPut

  if __FAMILIA==18
           bsf     TXSTA,TXEN            ; habilita o transmitter
           movwf   TXREG                 ; transmite o caracter para o receptor
serialPutW btfss   TXSTA,TRMT            ; transmissao terminou?
           goto    serialPutW            ; NAO --> espera terminar
           return
  endif

  if __FAMILIA==16
           bsf     STATUS, RP0           ; banco 1
           bsf     TXSTA, TXEN           ; habilita o transmitter
           bcf     STATUS, RP0           ; banco 1          
           movwf   TXREG                 ; transmite o caracter para o receptor
           bsf     STATUS, RP0           ; banco 1
serialPutW btfss   TXSTA,TRMT            ; transmissao terminou?
           goto    serialPutW            ; NAO --> espera terminar
           bcf     STATUS, RP0           ; banco 0
           return
  endif

; ---------------------------------------------------------------------------------------------
; serialWrite - Envia uma string ao terminal serial                                  27/11/2019

serialWrite
       if __FAMILIA==16
           return
       endif

       if __FAMILIA==18
           ; Envia string delimitada por 0x0D (ver macro SERIAL_WRITE)
           movff   TOSU,TBLPTRU              ; topo do stack para UPPER
           movff   TOSH,TBLPTRH              ; topo do stack para HIGH
           movff   TOSL,TBLPTRL              ; topo do stack para LOW
           call   _sendc                     ; vai imprimir
           btfsc   TBLPTRL,0                 ; endereco par???
           tblrd   *+                        ; NAO --> avanca para endereco par
           movf    TBLPTRU,w                 ; restaura o topo do stack
           movwf   TOSU                      ; restaura o topo do stack
           movf    TBLPTRH,w                 ; restaura o topo do stack
           movwf   TOSH                      ; restaura o topo do stack
           movf    TBLPTRL,w                 ; restaura o topo do stack
           movwf   TOSL                      ; restaura o topo do stack
           return                            ; fim
_sendc     tblrd   *+                        ; le um caracter na lista
	       movf    TABLAT, w                 ; pega o caracter lido
           xorlw   0x0D                      ; verifica o caracter
           btfsc   STATUS, Z                 ; delimitador de string?
           return                            ; SIM --> fim
           movf    TABLAT, w                 ; recerrega o caracter
           call    serialPut                 ; mostra o caracter
           bra     _sendc                    ; continua ate encontrar o delimitador
_fimstr    return                            ; fim
       endif


; ---------------------------------------------------------------------------------------------
; serialClear - Envia sequencia de apagamento da tela do terminal serial             27/11/2019
;
; Entrada: Nao ha
; Saida  : Nao ha
; ---------------------------------------------------------------------------------------------
serialClear
           movlw   0x1B
           call    serialPut
           movlw   "["
           call    serialPut
           movlw   "H"
           call    serialPut
           movlw   0x1B
           call    serialPut
           movlw   "["
           call    serialPut
           movlw   "J"
           call    serialPut
           movlw   0x00
           call    serialPut
           return

; ---------------------------------------------------------------------------------------------
; serialNewLine - Proxima linha no terminal serial                                   27/11/2019
; ---------------------------------------------------------------------------------------------
serialNewLine
           movlw 0x0D
           call  serialPut
           movlw 0x0A
           call  serialPut
           return
; ---------------------------------------------------------------------------------------------
; INTERRUPCOES: Rotina de tratamento da interrupcao (Rx) da porta serial.            27/11/2019
;
; A leitura da mensagem enviada pela Porta Serial sera feita atraves desta interrupcao.
;
; O buffer de leitura apontado por FSR/FSR1 deve ser grande o suficiente para conter a mensagem e
; pode residir em qualquer banco da memoria RAM visto que o acesso a ele sera feito 
; indiretamente atraves do registrador FSR/FSR1.
;
; Entrada: 1 - O registrador FSR/FSR1 deve apontar para o inicio de um buffer de leitura.
;          2 - O caracter delimitador de fim de string deve ser definido na macro SERIAL_OPEN.
;
; Saida  : 1 - O bit SERFLAGS<SEREOT> sera ligado indicando que ha nova mensagem no buffer.
;          2 - O buffer de leitura estara preenchido com caracteres enviados pela
;              Porta Serial inclusive o caracter delimitador de fim da mensagem.
;
; ---------------------------------------------------------------------------------------------
serPortISR movwf   SERWR               ; salva W
           movf    FSR, w              ; pega o indexador
           movwf   SERFSR              ; salva indexador
           swapf   STATUS,w            ; obtem o STATUS
           movwf   SERST               ; salva o STATUS
vfychar
           ; Verifica se houve erro de recepcao
           btfss   RCSTA, OERR         ; erro de recepcao???
           goto    norxerr             ; NAO --> segue normal
           bcf     RCSTA, CREN         ; SIM --> desliga recepcao para eliminar o erro
           nop                         ; delay para estabilizar
           nop                         ; delay para estabilizar
           nop                         ; delay para estabilizar
           bsf     RCSTA, CREN         ; religa a recepcao      

           ; Pega o caracter lido na porta serial
norxerr    movf    RCREG, w            ; le o caracter enviado pelo dispositivo
        if __FAMILIA==16
           movwf   INDF                ; armazena o caracter no endereco atual do buffer
           incf    FSR, f              ; avanca apontador do buffer
        else
           movwf   POSTINC1            ; armazena no buffer e avanca o apontador
        endif 

           ; Verifica se e' fim da string   
           xorwf   SERDLM, w           ; verifica fim da string
           btfsc   STATUS, Z           ; fim da string ?
           goto    eostr               ; SIM --> encerra interrupcao
           bcf     PIR1, RCIF          ; limpa flag da interrupcao

           ; Le a porta serial
wnxchar
           btfss   PIR1, RCIF          ; chegou o proximo caracter ?
           goto    wnxchar             ; NAI --> espera o proximo
           goto    vfychar             ; SIM --> vai verificar o caracter

           ; Restaura contexto
eostr      movf    SERFSR, w
           movwf   FSR
           swapf   SERST,w             ; recupera STATUS
           movwf   STATUS              ; restaura o STATUS antes da interrupcao
           swapf   SERWR,f             ; inverte nibles (LSB + MSB)
           swapf   SERWR,w             ; restaura W sem afetar o STATUS
           bsf     SERFLAGS, SEREOT
           retfie                      ; fim

           ENDM

